import uuid

def add_alias(config, username: str, email: str) -> dict:
    if config.get("aliases") is None:
        config["aliases"] = []

    try:
        config["aliases"].append(
            {"id": str(uuid.uuid4()), "username": username, "email": email}
        )
    except KeyError:
        config["aliases"] = [
            {"id": str(uuid.uuid4()), "username": username, "email": email}
        ]
    return config


import uuid

def check_user_exists(config, url: str, username: str, email: str, link_to: str) -> bool:
    user_tuple = (url, username, email, link_to)

    for user in config.get("users", []) if config.get("users", []) is not None else []:
        if (user["url"], user.get("username"), user.get("email"), user.get("linkTo")) == user_tuple:
            return True
    return False


def add_user(config, url: str, username: str, email: str, link_to: str) -> dict:
    user = {
        "id": str(uuid.uuid4()),
        "url": url,
        "current": False
    }
    if username and email:
        if username:
            user["username"] = username
        if email:
            user["email"] = email
    elif link_to:
        user["linkTo"] = link_to
    else:
        raise Exception("Invalid parameter")
    
    if config.get("users") is None:
        config["users"] = []

    try:
        config["users"].append(user)
    except KeyError:
        config["users"] = [user]
    return config


import os

import yaml

def save(config: dict):
    with open(
        os.path.join(os.path.join(os.path.expanduser("~"), ".parallel"), "config.yml"),
        "w",
        encoding="utf-8",
    ) as f:
        yaml.safe_dump(config, f, default_flow_style=False, allow_unicode=True)
    return config

def init_config():
    if os.path.isdir(os.path.join(os.path.expanduser("~"), ".parallel")):
        if os.path.isfile(
            os.path.join(os.path.join(os.path.expanduser("~"), ".parallel"), "config.yml")
        ):
            with open(
                os.path.join(
                    os.path.join(os.path.expanduser("~"), ".parallel"), "config.yml"
                ),
                "r",
                encoding="utf-8",
            ) as f:
                config = yaml.safe_load(f)
        else:
            config = {"v": 0, "users": [], "aliases": []}
            with open(
                os.path.join(
                    os.path.join(os.path.expanduser("~"), ".parallel"), "config.yml"
                ),
                "w",
                encoding="utf-8",
            ) as f:
                yaml.safe_dump(config, f)
            with open(
                os.path.join(
                    os.path.join(os.path.expanduser("~"), ".parallel"), "config.yml"
                ),
                "r",
                encoding="utf-8",
            ) as f:
                config = yaml.safe_load(f)
    else:
        os.mkdir(os.path.join(os.path.expanduser("~"), ".parallel"))
        config = {"v": 0, "users": [], "aliases": []}
        with open(
            os.path.join(os.path.join(os.path.expanduser("~"), ".parallel"), "config.yml"),
            "w",
            encoding="utf-8",
        ) as f:
            yaml.safe_dump(config, f)
        with open(
            os.path.join(os.path.join(os.path.expanduser("~"), ".parallel"), "config.yml"),
            "r",
            encoding="utf-8",
        ) as f:
            config = yaml.safe_load(f)
    return config


import os
import subprocess
import sys
from typing import Tuple


def find(executable_name):
    for dir in os.environ["PATH"].split(os.pathsep):
        path = os.path.join(dir, executable_name)
        if os.path.isfile(path):
            return path
    return None


def is_unixlike() -> Tuple[bool, str | None]:
    if sys.platform == "linux":
        return True, None
    elif sys.platform == "darwin":
        return True, None
    elif sys.platform == "aix":
        return True, None
    elif sys.platform == "android":
        return True, None
    elif sys.platform == "ios":
        return True, None
    elif sys.platform.startswith("freebsd"):
        return True, "freebsd"
    else:
        return False, None


def launch(env: dict):
    if sys.platform == "win32":
        pwsh_name = "pwsh.exe"
        powershell_name = "powershell.exe"
        cmd_name = "cmd.exe"

        pwsh_path = find(pwsh_name)
        if pwsh_path:
            subprocess.run(
                [
                    pwsh_path,
                    "-NoExit",
                    "-Command",
                    'function Prompt { $color = "Blue"; Write-Host "(parallel)" -ForegroundColor $color -NoNewLine; Write-Host " $PWD> " -NoNewLine; return " "; }',
                ],
                env=env,
            )
            return

        powershell_path = find(powershell_name)
        if powershell_path:
            subprocess.run(
                [
                    powershell_path,
                    "-NoExit",
                    "-Command",
                    'function Prompt { $color = "Blue"; Write-Host "(parallel)" -ForegroundColor $color -NoNewLine; Write-Host " $PWD> " -NoNewLine; return " "; }',
                ],
                env=env,
            )
            return

        cmd_path = find(cmd_name)
        if cmd_path:
            subprocess.run(
                [cmd_path, "/k", "prompt (parallel) $p$g"], env=env, shell=True
            )
            return
        
    elif is_unixlike()[0]:
        subprocess.run([os.environ.get("SHELL")])
        return